@@ -3,7 +3,7 @@
    Parser for dhclient config and lease files... */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: clparse.c,v 1.62.2.7 2004/11/24 17:39:14 dhankins Exp $ Copyright (c) 2004 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: clparse.c,v 1.67.76.2 2007/02/14 22:41:22 dhankins Exp $ Copyright (c) 2004-2007 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -173,7 +173,6 @@ int read_client_conf_file (const char *name, struct interface_info *ip,
 	status = (cfile -> warnings_occurred
 		  ? ISC_R_BADPARSE
 		  : ISC_R_SUCCESS);
-	close (file);
 	end_parse (&cfile);
 	return status;
 }
@@ -210,7 +209,6 @@ void read_client_leases ()
 
 	} while (1);
 
-	close (file);
 	end_parse (&cfile);
 }
 
@@ -241,7 +239,7 @@ void parse_client_statement (cfile, ip, config)
 {
 	int token;
 	const char *val;
-	struct option *option;
+	struct option *option = NULL;
 	struct executable_statement *stmt, **p;
 	enum statement_op op;
 	int lose;
@@ -290,7 +288,7 @@ void parse_client_statement (cfile, ip, config)
 		return;
 
 		/* REQUIRE can either start a policy statement or a
-		   comma-seperated list of names of required options. */
+		   comma-separated list of names of required options. */
 	      case REQUIRE:
 		next_token (&val, (unsigned *)0, cfile);
 		token = peek_token (&val, (unsigned *)0, cfile);
@@ -365,15 +363,15 @@ void parse_client_statement (cfile, ip, config)
 			return;
 		}
 
-		option = parse_option_name (cfile, 1, &known);
-		if (!option)
+		status = parse_option_name(cfile, 1, &known, &option);
+		if (status != ISC_R_SUCCESS || option == NULL)
 			return;
 
 		token = next_token (&val, (unsigned *)0, cfile);
 		if (token != CODE) {
 			parse_warn (cfile, "expecting \"code\" keyword.");
 			skip_to_semi (cfile);
-			free_option (option, MDL);
+			option_dereference(&option, MDL);
 			return;
 		}
 		if (ip) {
@@ -381,11 +379,11 @@ void parse_client_statement (cfile, ip, config)
 				    "option definitions may only appear in %s",
 				    "the outermost scope.");
 			skip_to_semi (cfile);
-			free_option (option, MDL);
+			option_dereference(&option, MDL);
 			return;
 		}
-		if (!parse_option_code_definition (cfile, option))
-			free_option (option, MDL);
+		parse_option_code_definition(cfile, option);
+		option_dereference(&option, MDL);
 		return;
 
 	      case MEDIA:
@@ -612,7 +610,8 @@ void parse_option_list (cfile, list)
 	int token;
 	const char *val;
 	pair p = (pair)0, q = (pair)0, r;
-	struct option *option;
+	struct option *option = NULL;
+	isc_result_t status;
 
 	ix = 0;
 	do {
@@ -627,8 +626,8 @@ void parse_option_list (cfile, list)
 			skip_to_semi (cfile);
 			return;
 		}
-		option = parse_option_name (cfile, 0, NULL);
-		if (!option) {
+		status = parse_option_name(cfile, 0, NULL, &option);
+		if (status != ISC_R_SUCCESS || option == NULL) {
 			parse_warn (cfile, "%s: expected option name.", val);
 			return;
 		}
@@ -637,12 +636,14 @@ void parse_option_list (cfile, list)
 				"%s.%s: Only global options allowed.",
 				option -> universe -> name, option->name );
 			skip_to_semi (cfile);
+			option_dereference(&option, MDL);
 			return;
 		}
 		r = new_pair (MDL);
 		if (!r)
 			log_fatal ("can't allocate pair for option code.");
 		r -> car = (caddr_t)(long)option -> code;
+		option_dereference(&option, MDL);
 		r -> cdr = (pair)0;
 		if (p)
 			q -> cdr = r;
@@ -774,7 +775,13 @@ int interface_or_dummy (struct interface_info **pi, const char *name)
 		if ((status = interface_allocate (&ip, MDL)) != ISC_R_SUCCESS)
 			log_fatal ("Can't record interface %s: %s",
 				   name, isc_result_totext (status));
-		strcpy (ip -> name, name);
+
+		if (strlen(name) >= sizeof(ip->name)) {
+			interface_dereference(&ip, MDL);
+			return 0;
+		}
+		strcpy(ip->name, name);
+
 		if (dummy_interfaces) {
 			interface_reference (&ip -> next,
 					     dummy_interfaces, MDL);
@@ -994,7 +1001,8 @@ void parse_client_lease_declaration (cfile, lease, ipp, clientp)
 			skip_to_semi (cfile);
 			break;
 		}
-		interface_or_dummy (ipp, val);
+		if (!interface_or_dummy (ipp, val))
+			log_fatal ("Can't allocate interface %s.", val);
 		break;
 
 	      case NAME:
@@ -1119,24 +1127,49 @@ void parse_reject_statement (cfile, config)
 {
 	int token;
 	const char *val;
-	struct iaddr addr;
-	struct iaddrlist *list;
+	struct iaddrmatch match;
+	struct iaddrmatchlist *list;
+	int i;
 
 	do {
-		if (!parse_ip_addr (cfile, &addr)) {
-			parse_warn (cfile, "expecting IP address.");
+		if (!parse_ip_addr_with_subnet (cfile, &match)) {
+			/* no warn: parser will have reported what's wrong */
 			skip_to_semi (cfile);
 			return;
 		}
 
-		list = (struct iaddrlist *)dmalloc (sizeof (struct iaddrlist),
-						    MDL);
+		/* check mask is not all zeros (because that would
+		 * reject EVERY address).  This check could be
+		 * simplified if we assume that the mask *always*
+		 * represents a prefix .. but perhaps it might be
+		 * useful to have a mask which is not a proper prefix
+		 * (perhaps for ipv6?).  The following is almost as
+		 * efficient as inspection of match.mask.iabuf[0] when
+		 * it IS a true prefix, and is more general when it is
+		 * not.
+		 */
+
+		for (i=0 ; i < match.mask.len ; i++) {
+		    if (match.mask.iabuf[i]) {
+			break;
+		    }
+		}
+
+		if (i == match.mask.len) {
+		    /* oops we found all zeros */
+		    parse_warn(cfile, "zero-length prefix is not permitted "
+				      "for reject statement");
+		    skip_to_semi(cfile);
+		    return;
+		} 
+
+		list = dmalloc(sizeof(struct iaddrmatchlist), MDL);
 		if (!list)
 			log_fatal ("no memory for reject list!");
 
-		list -> addr = addr;
-		list -> next = config -> reject_list;
-		config -> reject_list = list;
+		list->match = match;
+		list->next = config->reject_list;
+		config->reject_list = list;
 
 		token = next_token (&val, (unsigned *)0, cfile);
 	} while (token == COMMA);
