@@ -3,7 +3,7 @@
    DHCP/BOOTP Relay Agent. */
 
 /*
- * Copyright (c) 2004-2008 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1997-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char ocopyright[] =
-"$Id: dhcrelay.c,v 1.52.2.16 2008/05/14 20:54:24 dhankins Exp $ Copyright (c) 2004-2008 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: dhcrelay.c,v 1.59.2.4.6.1 2009/06/20 01:40:29 dhankins Exp $ Copyright (c) 2004-2009 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -66,6 +66,9 @@ int server_packets_relayed = 0;	/* Packets relayed from server to client. */
 int client_packet_errors = 0;	/* Errors sending packets to clients. */
 
 int add_agent_options = 0;	/* If nonzero, add relay agent options. */
+
+int agent_option_errors = 0;    /* Number of packets forwarded without
+				   agent options because there was no room. */
 int drop_agent_mismatches = 0;	/* If nonzero, drop server replies that
 				   don't have matching circuit-id's. */
 int corrupt_agent_options = 0;	/* Number of packets dropped because
@@ -80,7 +83,7 @@ int max_hop_count = 10;		/* Maximum hop count */
 
 
 	/* Maximum size of a packet with agent options added. */
-int dhcp_max_agent_option_packet_length = 576;
+int dhcp_max_agent_option_packet_length = DHCP_MTU_MIN;
 
 	/* What to do about packets we're asked to relay that
 	   already have a relay option: */
@@ -92,12 +95,13 @@ enum { forward_and_append,	/* Forward and append our own relay option. */
 u_int16_t local_port;
 u_int16_t remote_port;
 
+/* Relay agent server list. */
 struct server_list {
 	struct server_list *next;
 	struct sockaddr_in to;
 } *servers;
 
-static char copyright [] = "Copyright 2004-2008 Internet Systems Consortium.";
+static char copyright [] = "Copyright 2004-2009 Internet Systems Consortium.";
 static char arr [] = "All rights reserved.";
 static char message [] = "Internet Systems Consortium DHCP Relay Agent";
 static char url [] = "For info, please visit http://www.isc.org/sw/dhcp/";
@@ -106,23 +110,27 @@ int main (argc, argv, envp)
 	int argc;
 	char **argv, **envp;
 {
+	int fd;
 	int i;
 	struct servent *ent;
-	struct server_list *sp = (struct server_list *)0;
+	struct server_list *sp = NULL;
 	int no_daemon = 0;
 	int quiet = 0;
 	isc_result_t status;
 	char *s;
 
-	/* Make sure we have stdin, stdout and stderr. */
-	i = open ("/dev/null", O_RDWR);
-	if (i == 0)
-		i = open ("/dev/null", O_RDWR);
-	if (i == 1) {
-		i = open ("/dev/null", O_RDWR);
+	/* Make sure that file descriptors 0 (stdin), 1, (stdout), and
+	   2 (stderr) are open. To do this, we assume that when we
+	   open a file the lowest available file decriptor is used. */
+	fd = open("/dev/null", O_RDWR);
+	if (fd == 0)
+		fd = open("/dev/null", O_RDWR);
+	if (fd == 1)
+		fd = open("/dev/null", O_RDWR);
+	if (fd == 2)
 		log_perror = 0; /* No sense logging to /dev/null. */
-	} else if (i != -1)
-		close (i);
+	else if (fd != -1)
+		close(fd);
 
 #ifdef SYSLOG_4_2
 	openlog ("dhcrelay", LOG_NDELAY);
@@ -185,6 +193,10 @@ int main (argc, argv, envp)
 			if (++i == argc)
 				usage ();
 			dhcp_max_agent_option_packet_length = atoi (argv [i]);
+			if (dhcp_max_agent_option_packet_length > DHCP_MTU_MAX)
+				log_fatal("%s: packet length exceeds "
+					  "longest possible MTU\n",
+					  argv[i]);
 		} else if (!strcmp (argv [i], "-m")) {
 			if (++i == argc)
 				usage ();
@@ -642,7 +654,7 @@ int strip_relay_agent_options (in, out, packet, length)
 		length = sp - ((u_int8_t *)packet);
 
 		/* Make sure the packet isn't short (this is unlikely,
-                   but WTH) */
+		   but WTH) */
 		if (length < BOOTP_MIN_LEN) {
 			memset (sp, DHO_PAD, BOOTP_MIN_LEN - length);
 			length = BOOTP_MIN_LEN;
@@ -724,17 +736,15 @@ int find_interface_by_agent_option (packet, out, buf, len)
 	return -1;
 }
 
-/* Examine a packet to see if it's a candidate to have a Relay
-   Agent Information option tacked onto its tail.   If it is, tack
-   the option on.  */
-
-int add_relay_agent_options (ip, packet, length, giaddr)
-	struct interface_info *ip;
-	struct dhcp_packet *packet;
-	unsigned length;
-	struct in_addr giaddr;
-{
-	int is_dhcp = 0, agent_options_present = 0;
+/*
+ * Examine a packet to see if it's a candidate to have a Relay
+ * Agent Information option tacked onto its tail.   If it is, tack
+ * the option on.
+ */
+int
+add_relay_agent_options(struct interface_info *ip, struct dhcp_packet *packet,
+			unsigned length, struct in_addr giaddr) {
+	int is_dhcp = 0, mms;
 	unsigned optlen;
 	u_int8_t *op, *nextop, *sp, *max, *end_pad = NULL;
 
@@ -748,7 +758,7 @@ int add_relay_agent_options (ip, packet, length, giaddr)
 	if (memcmp(packet->options, DHCP_OPTIONS_COOKIE, 4))
 		return length;
 
-	max = ((u_int8_t *)packet) + length;
+	max = ((u_int8_t *)packet) + dhcp_max_agent_option_packet_length;
 
 	/* Commence processing after the cookie. */
 	sp = op = &packet->options[4];
@@ -780,7 +790,17 @@ int add_relay_agent_options (ip, packet, length, giaddr)
 		      case DHO_DHCP_MESSAGE_TYPE:
 			is_dhcp = 1;
 			goto skip;
-			break;
+
+			/*
+			 * If there's a maximum message size option, we
+			 * should pay attention to it
+			 */
+		      case DHO_DHCP_MAX_MESSAGE_SIZE:
+			mms = ntohs(*(op + 2));
+			if (mms < dhcp_max_agent_option_packet_length &&
+			    mms >= DHCP_MTU_MIN)
+				max = ((u_int8_t *)packet) + mms;
+			goto skip;
 
 			/* Quit immediately if we hit an End option. */
 		      case DHO_END:
@@ -874,30 +894,45 @@ int add_relay_agent_options (ip, packet, length, giaddr)
 		log_fatal ("total agent option length (%u) out of range "
 			   "[3 - 255] on %s\n", optlen, ip->name);
 
-	/* Is there room for the option, its code+len, and DHO_END? */
-	if ((sp > max) || (max - sp < optlen + 3))
-		return 0;
-
-	/* Okay, cons up *our* Relay Agent Information option. */
-	*sp++ = DHO_DHCP_AGENT_OPTIONS;
-	*sp++ = optlen;
-
-	/* Copy in the circuit id... */
-	*sp++ = RAI_CIRCUIT_ID;
-	*sp++ = ip->circuit_id_len;
-	memcpy(sp, ip->circuit_id, ip->circuit_id_len);
-	sp += ip->circuit_id_len;
-
-	/* Copy in remote ID... */
-	if (ip->remote_id) {
-		*sp++ = RAI_REMOTE_ID;
-		*sp++ = ip->remote_id_len;
-		memcpy(sp, ip->remote_id, ip->remote_id_len);
-		sp += ip->remote_id_len;
-	}
-
-	/* Deposit an END option. */
-	*sp++ = DHO_END;
+	/*
+ 	 * Is there room for the option, its code+len, and DHO_END?
+ 	 * If not, forward without adding the option.
+ 	 */
+ 	if (max - sp >= optlen + 3) {
+ 		log_debug("Adding %d-byte relay agent option", optlen + 3);
+ 
+ 		/* Okay, cons up *our* Relay Agent Information option. */
+ 		*sp++ = DHO_DHCP_AGENT_OPTIONS;
+ 		*sp++ = optlen;
+ 
+ 		/* Copy in the circuit id... */
+ 		*sp++ = RAI_CIRCUIT_ID;
+ 		*sp++ = ip->circuit_id_len;
+ 		memcpy(sp, ip->circuit_id, ip->circuit_id_len);
+ 		sp += ip->circuit_id_len;
+ 
+ 		/* Copy in remote ID... */
+ 		if (ip->remote_id) {
+ 			*sp++ = RAI_REMOTE_ID;
+ 			*sp++ = ip->remote_id_len;
+ 			memcpy(sp, ip->remote_id, ip->remote_id_len);
+ 			sp += ip->remote_id_len;
+ 		}
+ 	} else {
+		 ++agent_option_errors;
+ 		log_error("No room in packet (used %d of %d) "
+ 			  "for %d-byte relay agent option: omitted",
+ 			   (int) (sp - ((u_int8_t *) packet)),
+ 			   (int) (max - ((u_int8_t *) packet)),
+ 			   optlen + 3);
+ 	}
+ 
+ 	/*
+ 	 * Deposit an END option unless the packet is full (shouldn't
+ 	 * be possible).
+ 	 */
+ 	if (sp < max)
+ 		*sp++ = DHO_END;
 
 	/* Recalculate total packet length. */
 	length = sp - ((u_int8_t *)packet);
