@@ -3,7 +3,7 @@
    Server-specific in-memory database support. */
 
 /*
- * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2008 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: mdb.c,v 1.83.16.7 2007/06/08 18:57:02 dhankins Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: mdb.c,v 1.83.16.10 2008/01/22 19:02:51 dhankins Exp $ Copyright (c) 2004-2008 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -533,7 +533,6 @@ void new_address_range (cfile, low, high, subnet, pool, lpchain)
 	struct lease **lpchain;
 {
 	struct lease *address_range, *lp, *plp;
-	struct iaddr net;
 	unsigned min, max, i;
 	char lowbuf [16], highbuf [16], netbuf [16];
 	struct shared_network *share = subnet -> shared_network;
@@ -563,23 +562,19 @@ void new_address_range (cfile, low, high, subnet, pool, lpchain)
 			log_fatal ("Can't allocate lease/hw hash");
 	}
 
-	/* Make sure that high and low addresses are in same subnet. */
-	net = subnet_number (low, subnet -> netmask);
-	if (!addr_eq (net, subnet_number (high, subnet -> netmask))) {
-		strcpy (lowbuf, piaddr (low));
-		strcpy (highbuf, piaddr (high));
-		strcpy (netbuf, piaddr (subnet -> netmask));
-		log_fatal ("Address range %s to %s, netmask %s spans %s!",
-		       lowbuf, highbuf, netbuf, "multiple subnets");
+	/* Make sure that high and low addresses are in this subnet. */
+	if (!addr_eq(subnet->net, subnet_number(low, subnet->netmask))) {
+		strcpy(lowbuf, piaddr(low));
+		strcpy(netbuf, piaddr(subnet->net));
+		log_fatal("bad range, address %s not in subnet %s netmask %s",
+			  lowbuf, netbuf, piaddr(subnet->netmask));
 	}
 
-	/* Make sure that the addresses are on the correct subnet. */
-	if (!addr_eq (net, subnet -> net)) {
-		strcpy (lowbuf, piaddr (low));
-		strcpy (highbuf, piaddr (high));
-		strcpy (netbuf, piaddr (subnet -> netmask));
-		log_fatal ("Address range %s to %s not on net %s/%s!",
-		       lowbuf, highbuf, piaddr (subnet -> net), netbuf);
+	if (!addr_eq(subnet->net, subnet_number(high, subnet->netmask))) {
+		strcpy(highbuf, piaddr(high));
+		strcpy(netbuf, piaddr(subnet->net));
+		log_fatal("bad range, address %s not in subnet %s netmask %s",
+			  highbuf, netbuf, piaddr(subnet->netmask));
 	}
 
 	/* Get the high and low host addresses... */
@@ -2330,13 +2325,13 @@ void expire_all_pools ()
 				if (l->binding_state == FTS_FREE) {
 					if (i == FREE_LEASES)
 						p->free_leases++;
-					else
+					else if (i != RESERVED_LEASES)
 						log_fatal("Impossible case "
 							  "at %s:%d.", MDL);
 				} else if (l->binding_state == FTS_BACKUP) {
 					if (i == BACKUP_LEASES)
 						p->backup_leases++;
-					else
+					else if (i != RESERVED_LEASES)
 						log_fatal("Impossible case "
 							  "at %s:%d.", MDL);
 				}
@@ -2565,7 +2560,7 @@ void free_everything ()
 		    pool_reference (&pn, nc -> pools, MDL);
 		    do {
 			struct lease **lptr[RESERVED_LEASES+1];
-			
+
 			if (pn) {
 			    pool_reference (&pc, pn, MDL);
 			    pool_dereference (&pn, MDL);
@@ -2574,7 +2569,7 @@ void free_everything ()
 			    pool_reference (&pn, pc -> next, MDL);
 			    pool_dereference (&pc -> next, MDL);
 			}
-			
+
 			lptr [FREE_LEASES] = &pc -> free;
 			lptr [ACTIVE_LEASES] = &pc -> active;
 			lptr [EXPIRED_LEASES] = &pc -> expired;
